@@ -14,34 +14,48 @@ import os
 import shutil
 import subprocess
 import time
+import uuid
 from hashlib import md5
 
 import click
 import requests
 
-from reana_commons.config import (CVMFS_REPOSITORIES, REANA_CVMFS_PVC_TEMPLATE,
-                                  REANA_CVMFS_SC_TEMPLATE)
+from reana_commons.config import (
+    CVMFS_REPOSITORIES,
+    REANA_COMPONENT_NAMING_SCHEME,
+    REANA_COMPONENT_PREFIX,
+    REANA_COMPONENT_TYPES,
+    REANA_CVMFS_PVC_TEMPLATE,
+    REANA_CVMFS_SC_TEMPLATE,
+    HTCONDOR_JOB_FLAVOURS,
+)
 
 
 def click_table_printer(headers, _filter, data):
     """Generate space separated output for click commands."""
     _filter = [h.lower() for h in _filter] + [h.upper() for h in _filter]
+    header_indexes = [i for i, item in enumerate(headers)]
+    if _filter:
+        header_indexes = [
+            i for i, item in enumerate(headers) if item.upper() in _filter
+        ]
     headers = [h for h in headers if not _filter or h in _filter]
     # Maximum header width
     header_widths = [len(h) for h in headers]
 
     for row in data:
-        for idx in range(len(headers)):
+        for i, idx in enumerate(header_indexes):
             # If a row contains an element which is wider update maximum width
-            if header_widths[idx] < len(str(row[idx])):
-                header_widths[idx] = len(str(row[idx]))
+            if header_widths[i] < len(str(row[idx])):
+                header_widths[i] = len(str(row[idx]))
     # Prepare the format string with the maximum widths
-    formatted_output_parts = ['{{:<{0}}}'.format(hw)
-                              for hw in header_widths]
-    formatted_output = '   '.join(formatted_output_parts)
+    formatted_output_parts = ["{{:<{0}}}".format(hw) for hw in header_widths]
+    formatted_output = "   ".join(formatted_output_parts)
     # Print the table with the headers capitalized
     click.echo(formatted_output.format(*[h.upper() for h in headers]))
     for row in data:
+        if header_indexes:
+            row = [row[i] for i in header_indexes]
         click.echo(formatted_output.format(*row))
 
 
@@ -58,7 +72,7 @@ def calculate_hash_of_dir(directory, file_list=None):
                 if file_list is not None and file_path not in file_list:
                     continue
                 try:
-                    _file_object = open(file_path, 'rb')
+                    _file_object = open(file_path, "rb")
                 except Exception:
                     # You can't open the file for some reason
                     _file_object.close()
@@ -80,11 +94,11 @@ def calculate_hash_of_dir(directory, file_list=None):
 
 def calculate_job_input_hash(job_spec, workflow_json):
     """Calculate md5 hash of job specification and workflow json."""
-    if 'workflow_workspace' in job_spec:
-        del job_spec['workflow_workspace']
+    if "workflow_workspace" in job_spec:
+        del job_spec["workflow_workspace"]
     job_md5_buffer = md5()
-    job_md5_buffer.update(json.dumps(job_spec).encode('utf-8'))
-    job_md5_buffer.update(json.dumps(workflow_json).encode('utf-8'))
+    job_md5_buffer.update(json.dumps(job_spec).encode("utf-8"))
+    job_md5_buffer.update(json.dumps(workflow_json).encode("utf-8"))
     return job_md5_buffer.hexdigest()
 
 
@@ -100,33 +114,30 @@ def calculate_file_access_time(workflow_workspace):
 
 def copy_openapi_specs(output_path, component):
     """Copy generated and validated openapi specs to reana-commons module."""
-    if component == 'reana-server':
-        file = 'reana_server.json'
-    elif component == 'reana-workflow-controller':
-        file = 'reana_workflow_controller.json'
-    elif component == 'reana-job-controller':
-        file = 'reana_job_controller.json'
-    if os.environ.get('REANA_SRCDIR'):
-        reana_srcdir = os.environ.get('REANA_SRCDIR')
+    if component == "reana-server":
+        file = "reana_server.json"
+    elif component == "reana-workflow-controller":
+        file = "reana_workflow_controller.json"
+    elif component == "reana-job-controller":
+        file = "reana_job_controller.json"
+    if os.environ.get("REANA_SRCDIR"):
+        reana_srcdir = os.environ.get("REANA_SRCDIR")
     else:
-        reana_srcdir = os.path.join('..')
+        reana_srcdir = os.path.join("..")
     try:
         reana_commons_specs_path = os.path.join(
-            reana_srcdir,
-            'reana-commons',
-            'reana_commons',
-            'openapi_specifications')
+            reana_srcdir, "reana-commons", "reana_commons", "openapi_specifications"
+        )
         if os.path.exists(reana_commons_specs_path):
             if os.path.isfile(output_path):
-                shutil.copy(output_path,
-                            os.path.join(reana_commons_specs_path,
-                                         file))
+                shutil.copy(output_path, os.path.join(reana_commons_specs_path, file))
                 # copy openapi specs file as well to docs
-                shutil.copy(output_path,
-                            os.path.join('docs', 'openapi.json'))
+                shutil.copy(output_path, os.path.join("docs", "openapi.json"))
     except Exception as e:
-        click.echo('Something went wrong, could not copy openapi '
-                   'specifications to reana-commons \n{0}'.format(e))
+        click.echo(
+            "Something went wrong, could not copy openapi "
+            "specifications to reana-commons \n{0}".format(e)
+        )
 
 
 def get_workflow_status_change_verb(status):
@@ -134,60 +145,60 @@ def get_workflow_status_change_verb(status):
 
     :param status: String which represents the status the workflow changed to.
     """
-    verb = ''
-    if status.endswith('ing'):
-        verb = 'is'
-    elif status.endswith('ed'):
-        verb = 'has been'
+    verb = ""
+    if status.endswith("ing"):
+        verb = "is"
+    elif status.endswith("ed"):
+        verb = "has been"
     else:
-        raise ValueError('Unrecognised status {}'.format(status))
+        raise ValueError("Unrecognised status {}".format(status))
 
     return verb
 
 
-def build_progress_message(total=None,
-                           running=None,
-                           finished=None,
-                           failed=None,
-                           cached=None):
+def build_progress_message(
+    total=None, running=None, finished=None, failed=None, cached=None
+):
     """Build the progress message with correct formatting."""
     progress_message = {}
     if total:
-        progress_message['total'] = total
+        progress_message["total"] = total
     if running:
-        progress_message['running'] = running
+        progress_message["running"] = running
     if finished:
-        progress_message['finished'] = finished
+        progress_message["finished"] = finished
     if failed:
-        progress_message['failed'] = failed
+        progress_message["failed"] = failed
     if cached:
-        progress_message['cached'] = cached
+        progress_message["cached"] = cached
     return progress_message
 
 
-def build_caching_info_message(job_spec,
-                               job_id,
-                               workflow_workspace,
-                               workflow_json,
-                               result_path):
+def build_caching_info_message(
+    job_spec, job_id, workflow_workspace, workflow_json, result_path
+):
     """Build the caching info message with correct formatting."""
     caching_info_message = {
         "job_spec": job_spec,
         "job_id": job_id,
         "workflow_workspace": workflow_workspace,
         "workflow_json": workflow_json,
-        "result_path": result_path
+        "result_path": result_path,
     }
     return caching_info_message
 
 
-def get_workspace_disk_usage(workspace, summarize=False):
+def get_workspace_disk_usage(workspace, summarize=False, block_size=None):
     """Retrieve disk usage information of a workspace."""
-    command = ['du', '-h']
-    if summarize:
-        command.append('-s')
+    command = ["du"]
+    if block_size in ["b", "k"]:
+        command.append("-{}".format(block_size))
     else:
-        command.append('-a')
+        command.append("-h")
+    if summarize:
+        command.append("-s")
+    else:
+        command.append("-a")
     command.append(workspace)
     disk_usage_info = subprocess.check_output(command).decode().split()
     # create pairs of (size, filename)
@@ -196,8 +207,7 @@ def get_workspace_disk_usage(workspace, summarize=False):
     for filesize_pair in filesize_pairs:
         size, name = filesize_pair
         # trim workspace path in every file name
-        filesizes.append({'name': name[len(workspace):],
-                          'size': size})
+        filesizes.append({"name": name[len(workspace) :], "size": size})
     return filesizes
 
 
@@ -205,8 +215,8 @@ def render_cvmfs_pvc(cvmfs_volume):
     """Render REANA_CVMFS_PVC_TEMPLATE."""
     name = CVMFS_REPOSITORIES[cvmfs_volume]
     rendered_template = dict(REANA_CVMFS_PVC_TEMPLATE)
-    rendered_template['metadata']['name'] = 'csi-cvmfs-{}-pvc'.format(name)
-    rendered_template['spec']['storageClassName'] = "csi-cvmfs-{}".format(name)
+    rendered_template["metadata"]["name"] = "csi-cvmfs-{}-pvc".format(name)
+    rendered_template["spec"]["storageClassName"] = "csi-cvmfs-{}".format(name)
     return rendered_template
 
 
@@ -214,8 +224,8 @@ def render_cvmfs_sc(cvmfs_volume):
     """Render REANA_CVMFS_SC_TEMPLATE."""
     name = CVMFS_REPOSITORIES[cvmfs_volume]
     rendered_template = dict(REANA_CVMFS_SC_TEMPLATE)
-    rendered_template['metadata']['name'] = "csi-cvmfs-{}".format(name)
-    rendered_template['parameters']['repository'] = cvmfs_volume
+    rendered_template["metadata"]["name"] = "csi-cvmfs-{}".format(name)
+    rendered_template["parameters"]["repository"] = cvmfs_volume
     return rendered_template
 
 
@@ -225,10 +235,9 @@ def create_cvmfs_storage_class(cvmfs_volume):
     from reana_commons.k8s.api_client import current_k8s_storagev1_api_client
 
     try:
-        current_k8s_storagev1_api_client.\
-            create_storage_class(
-                render_cvmfs_sc(cvmfs_volume)
-            )
+        current_k8s_storagev1_api_client.create_storage_class(
+            render_cvmfs_sc(cvmfs_volume)
+        )
     except ApiException as e:
         if e.status != 409:
             raise e
@@ -240,11 +249,9 @@ def create_cvmfs_persistent_volume_claim(cvmfs_volume):
     from reana_commons.k8s.api_client import current_k8s_corev1_api_client
 
     try:
-        current_k8s_corev1_api_client.\
-            create_namespaced_persistent_volume_claim(
-                "default",
-                render_cvmfs_pvc(cvmfs_volume)
-            )
+        current_k8s_corev1_api_client.create_namespaced_persistent_volume_claim(
+            "default", render_cvmfs_pvc(cvmfs_volume)
+        )
     except ApiException as e:
         if e.status != 409:
             raise e
@@ -255,14 +262,15 @@ def format_cmd(cmd):
     if isinstance(cmd, str):
         cmd = [cmd]
     elif not isinstance(cmd, list):
-        raise ValueError('Command should be a list or a string and not {}'
-                         .format(type(cmd)))
+        raise ValueError(
+            "Command should be a list or a string and not {}".format(type(cmd))
+        )
     return cmd
 
 
 def check_connection_to_job_controller(port=5000):
     """Check connection from workflow engine to job controller."""
-    url = 'http://localhost:{}/jobs'.format(port)
+    url = "http://localhost:{}/jobs".format(port)
     retry_counter = 0
     while retry_counter < 5:
         try:
@@ -274,4 +282,63 @@ def check_connection_to_job_controller(port=5000):
         time.sleep(10)
         retry_counter += 1
     else:
-        logging.error('Job controller is not reachable.', exc_info=True)
+        logging.error("Job controller is not reachable.", exc_info=True)
+
+
+def build_unique_component_name(component_type, id=None):
+    """Use REANA component type and id build a human readable component name.
+
+    :param component_type: One of
+        ``reana_commons.config.REANA_COMPONENT_TYPES``.
+    :param id: Unique identifier, if not specified a new UUID4 is created.
+
+    :return: String representing the component name, i.e. reana-run-job-123456.
+    """
+    if component_type not in REANA_COMPONENT_TYPES:
+        raise ValueError(
+            "{} not valid component type.\nChoose one of: {}".format(
+                component_type, REANA_COMPONENT_TYPES
+            )
+        )
+
+    return REANA_COMPONENT_NAMING_SCHEME.format(
+        prefix=REANA_COMPONENT_PREFIX,
+        component_type=component_type,
+        id=id or str(uuid.uuid4()),
+    )
+
+
+def check_htcondor_max_runtime(specification):
+    """Check if the field htcondor_max_runtime has a valid input.
+
+    :param reana_specification: reana specification of workflow.
+    """
+    check_pass = True
+    steps_with_max_runtime = (
+        step
+        for step in specification["steps"]
+        if step.get("htcondor_max_runtime", False)
+    )
+    for i, step in enumerate(steps_with_max_runtime):
+        htcondor_max_runtime = step["htcondor_max_runtime"]
+        if (
+            not str.isdigit(htcondor_max_runtime)
+            and not htcondor_max_runtime in HTCONDOR_JOB_FLAVOURS
+        ):
+            check_pass = False
+            click.secho(
+                "In step {0}:\n'{1}' is not a valid input for htcondor_max_runtime. Inputs must be a digit in the form of a string, or one of the following job flavours: '{2}'.".format(
+                    step.get("name", i),
+                    htcondor_max_runtime,
+                    "' '".join(
+                        [
+                            k
+                            for k, v in sorted(
+                                HTCONDOR_JOB_FLAVOURS.items(), key=lambda key: key[1]
+                            )
+                        ]
+                    ),
+                ),
+                fg="red",
+            )
+    return check_pass
